﻿/*
 * PROJEKT: AmbiLight 3
 * ====================
 *
 * Created: 26.10.2012 17:30:56
 *  Author: Christoph Söllner
 *    Desc: Bewährter XMega-AVR-Controller, aber diesmal mit einem RFM70 (Pollin oder eBay) ausgestattet und wireless.
 *
 */ 

#define led_error_on()  PORTA.OUTSET=PIN1_bm;
#define led_error_off() PORTA.OUTCLR=PIN1_bm;
#define led_ok_on()     PORTA.OUTSET=PIN0_bm;
#define led_ok_off()    PORTA.OUTCLR=PIN0_bm;

/***********************************
 ** INCLUDES                      **
 ***********************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdio.h>
#include "config.h"
#include "cs_rfm70.h"
#include "flash.h"

/***********************************
 ** GLOBAL VARIABLES              **
 ***********************************/
unsigned char rxBaseAddress[5] = { 0x20, 0x20, 0x20, 0x20, 0xFF };
unsigned char rxBaseAddressLsbNew = 0x00;
volatile unsigned char rxCompleteInterrupt = 0;
unsigned char rxBuffer[30];


int main(void) {
	unsigned char i;
	
	// ******* PORT? INIT
	PORTA.DIR = PIN0_bm | PIN1_bm | PIN2_bm | PIN3_bm | PIN4_bm;
	PORTA.PIN5CTRL = PORT_OPC_PULLUP_gc;
	PORTA.PIN6CTRL = PORT_OPC_PULLUP_gc;
	PORTA.PIN7CTRL = PORT_OPC_PULLUP_gc;
	led_error_on();

	PORTC.OUT = 0;
	PORTC.DIR =         PIN4_bm|PIN3_bm|PIN2_bm|PIN1_bm|PIN0_bm;
	PORTD.OUT = 0;
	PORTD.DIR = PIN5_bm|PIN4_bm|PIN3_bm|PIN2_bm|PIN1_bm|PIN0_bm;
	PORTE.OUT = 0;
	PORTE.DIR =                 PIN3_bm|PIN2_bm|PIN1_bm|PIN0_bm;

	// ******* CLOCK INIT
	OSC.CTRL   = (1<<OSC_RC32MEN_bp);
	while (!(OSC.STATUS & OSC_RC32MRDY_bm));
	CCP        = CCP_IOREG_gc;
	CLK.CTRL   = CLK_SCLKSEL_RC32M_gc; // CSource Clock Extern
	CCP        = CCP_IOREG_gc;         // Conf.Änderung erlauben
	CLK.PSCTRL = CLK_PSBCDIV_1_1_gc;   // ClockDiv 1, also volle 32MHz
	led_error_off();
	
	// ******* TIMER INIT
	TCC0.CTRLA = TC_CLKSEL_DIV1_gc;
	TCC0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCC0.PER   = 0xFFFF;
	TCC1.CTRLA = TC_CLKSEL_DIV1_gc;
	TCC1.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCC1.PER   = 0xFFFF;

	TCD0.CTRLA = TC_CLKSEL_DIV1_gc;
	TCD0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCD0.PER   = 0xFFFF;
	TCD1.CTRLA = TC_CLKSEL_DIV1_gc;
	TCD1.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCD1.PER   = 0xFFFF;

	TCE0.CTRLA = TC_CLKSEL_DIV1_gc;
	TCE0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCE0.PER   = 0xFFFF;

	// ******* RFM Init
	rfm70_initPortsAndSpi();
	rfm70_initChipRegisters();
	rfm70_selectChannel(40);
	rfm70_setPipeAddress(RFM70_REG_RX_ADDR_P0, &rxBaseAddress[0], 5);
	rfm70_selectReceiveMode();

	sei();
	// ####### APP INIT: 
	// - Wait ms* ADDR_NIBBLE
	// - Send ID
	// - For 10 Seconds after Power-Up:
	//   "N"=New Channel

    while(1) {
		// TASK: Read Address nibble and write new Address if changed.
		rxBaseAddressLsbNew = (((~PORTA_IN)&0b11100000)>>5);

		if (rxBaseAddress[4] != rxBaseAddressLsbNew) {
			led_error_on();
			_delay_ms(500);
			rxBaseAddressLsbNew = (((~PORTA_IN)&0b11100000)>>5);
			rxBaseAddress[4] = rxBaseAddressLsbNew;
			rfm70_setPipeAddress(RFM70_REG_RX_ADDR_P0, &rxBaseAddress[0], 5);
			led_error_off();
		}

		// TASK: When Interrupt,
		if (rxCompleteInterrupt) {
			//       - SwitchOn Yellow LED
			led_ok_on();
			//       - Arm Interrupt for new Run
			rxCompleteInterrupt = 0;
			//       - Read new Paket (Size, Queue and Data) into
			rfm70_readFromFifoIntoBuffer(rxBuffer, 15);
			//       - Clear RX-INT Bit
			i = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_REG|RFM70_REG_STATUS);
			i |= 0b01000000;
			rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_STATUS, i);
			//       - SwitchOff Yellow LED
			_delay_ms(1);
			led_error_off();
			led_ok_off();
		}

		// TASK: Update Buffered TCC*-Register with PWM
		//       Layout R0R1R2R3R4G0G1G2G3G4B0B1B2B3B4
		TCE0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[ 0]]); // C1:RED
		TCE0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[ 5]]); // C1:GREEN
		TCE0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[10]]); // C1:BLUE

		TCD1_CCBBUF = pgm_read_word(&gamma22[rxBuffer[ 1]]); // C2:RED
		TCE0_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 6]]); // C2:GREEN
		TCD1_CCABUF = pgm_read_word(&gamma22[rxBuffer[11]]); // C2:BLUE

		TCD0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[ 2]]); // C3:RED
		TCD0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[ 7]]); // C3:GREEN
		TCD0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[12]]); // C3:BLUE

		TCC1_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 3]]); // C4:RED
		TCD0_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 8]]); // C4:GREEN
		TCC0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[13]]); // C4:BLUE

		TCC0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[ 4]]); // C5:RED
		TCC0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[ 9]]); // C5:GREEN
		TCC0_CCABUF = pgm_read_word(&gamma22[rxBuffer[14]]); // C5:BLUE
    }
}

ISR(PORTB_INT0_vect) { rxCompleteInterrupt = 1; }