﻿/*
 * cs_rfm70.c
 *
 * Created: 29.10.2012 14:05:00
 *  Author: Christoph Söllner
 */ 

#include "cs_rfm70.h"

//Bank0 register initialization value
const unsigned char rfm70_initBank0_R0_R1D[][2] PROGMEM = {
	{0, 0b00111111 }, // CS:
	{1,0x00}, // Enable auto acknowledgement data pipe5\4\3\2\1\0
	{2,0x03}, // Enable RX Addresses pipe5\4\3\2\1\0
	{3,0x03}, // RX/TX address field width 5byte
	{4,0xff}, // auto retransmission dalay (4000us),auto retransmission count(15)
	{5,0x17}, // 23 channel
	{6,0x3F}, // air data rate-1M,out power 0dbm,setup LNA gain
	{7,0x07}, //
	{8,0x00}, //
	{9,0x00},
	{12,0xc3},//only LSB Receive address data pipe 2, MSB bytes is equal to RX_ADDR_P1[39:8]
	{13,0xc4},//only LSB Receive address data pipe 3, MSB bytes is equal to RX_ADDR_P1[39:8]
	{14,0xc5},//only LSB Receive address data pipe 4, MSB bytes is equal to RX_ADDR_P1[39:8]
	{15,0xc6},//only LSB Receive address data pipe 5, MSB bytes is equal to RX_ADDR_P1[39:8]
	{17,0x0F},//Number of bytes in RX payload in data pipe0(32 byte) 
	{18,0x0F},//Number of bytes in RX payload in data pipe1(32 byte)
	{19,0x0F},//Number of bytes in RX payload in data pipe2(32 byte)
	{20,0x0F},//Number of bytes in RX payload in data pipe3(32 byte)
	{21,0x0F},//Number of bytes in RX payload in data pipe4(32 byte)
	{22,0x0F},//Number of bytes in RX payload in data pipe5(32 byte)
	{23,0x00},//fifo status
	{28,0x3F},//Enable dynamic payload destinationBufferLength data pipe5\4\3\2\1\0
	{29,0b00000111}//Enables Dynamic Payload Length,Enables Payload with ACK,Enables the W_TX_PAYLOAD_NOACK command 
};

const unsigned char rfm70_initBank1_R0_R0D[][4] PROGMEM = {
	{0x40, 0x4B, 0x01, 0xE2}, // DS: 0x404B01E2 (MSBYTE WRITTEN FIRST!)
	{0xC0, 0x4B, 0x00, 0x00}, // DS: 0xC04B0000 (MSBYTE WRITTEN FIRST!)
	{0xD0, 0xFC, 0x8C, 0x02}, // DS: 0xD0FC8C02 (MSBYTE WRITTEN FIRST!)
	{0x99, 0x00, 0x39, 0x41}, // DS: 0x99003941 (MSBYTE WRITTEN FIRST!)
	{0xb9, 0x9E, 0x86, 0x0B}, // DS: 0xD99E860B (for high Power), 0xD98E860B (for -30db TX output power)  (MSBYTE WRITTEN FIRST!)
	{0x24, 0x06, 0x7F, 0xA6}, // DS:  (MSBYTE WRITTEN FIRST!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN FIRST!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN FIRST!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN FIRST!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN last!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN last!)
	{0x00, 0x00, 0x00, 0x00}, // DS: 0 (MSBYTE WRITTEN last!)
	{0x00, 0x12, 0x73, 0x00}, // DS: 0x00731200 (MSBYTE WRITTEN last!)
	{0x36, 0xB4, 0x80, 0x00}  // DS: 0x0080B436 (MSBYTE WRITTEN last!)
};

// 11 Bytes for register 0E, taken from datasheet (MSBYTE WRITTEN last!)
const unsigned char rfm70_initBank1_R0E[] PROGMEM = {
	0x41, 0x20, 0x08, 0x04, 0x81, 0x20, 0xCF, 0xF7, 0xFE, 0xFF, 0xFF
};

// Addresses for RX-Pipes
const unsigned char RX0_Address[] PROGMEM = { 0x34, 0x43, 0x10, 0x10, 0x01 }; // Receive address data pipe 0
const unsigned char RX1_Address[] PROGMEM = { 0x34, 0x43, 0x10, 0x10, 0x02 }; // Receive address data pipe 1

// ***************************************************************************
//                  RFM70 LOW LEVEL COMMUNICATION FUNCTIONS
// ***************************************************************************
void rfm70_writeSingleByteToRegister(unsigned char destinationRegister, unsigned char value) {
	rfm70_spi_slaveSelect();
	rfm70_spi_rw(destinationRegister);
	rfm70_spi_rw(value);
	rfm70_spi_slaveUnselect();
}                                                         
unsigned char rfm70_readSingleByteFromRegister(unsigned char destinationRegister) {                                                           
	unsigned char value;
	rfm70_spi_slaveSelect();
	rfm70_spi_rw(destinationRegister);
	value = rfm70_spi_rw(0);
	rfm70_spi_slaveUnselect();
	return(value);
}                                                           
void rfm70_readIntoBufferFromRegisterN(unsigned char rfm70DestinationRegister, unsigned char *destinationBuffer, unsigned char destinationBufferLength) {                                                           
	unsigned char i;                              
	rfm70_spi_slaveSelect();
	rfm70_spi_rw(rfm70DestinationRegister);
	for(i=0; i<destinationBufferLength; i++) destinationBuffer[i] = rfm70_spi_rw(0);
	rfm70_spi_slaveUnselect();               
}                         
void rfm70_writeToRegisterFromBufferN(unsigned char rfm70DestinationRegister, unsigned char *sourceBuffer, unsigned char sourceBufferLength) {                                                           
	unsigned char i;                              
	rfm70_spi_slaveSelect();
	rfm70_spi_rw(rfm70DestinationRegister);
	for(i=0; i<sourceBufferLength; i++) rfm70_spi_rw(sourceBuffer[i]);
	rfm70_spi_slaveUnselect();
}

// ***************************************************************************
//                  RFM70 DEVICE CONFIG FUNCTIONS
// ***************************************************************************
void rfm70_selectReceiveMode() {
	unsigned char value;
	rfm70_writeSingleByteToRegister(RFM70_CMD_FLUSH_RX, 0);                        // flush Rx
	value = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_REG|RFM70_REG_STATUS); // read register STATUS's value
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_STATUS, value);  // clear RX_DR or TX_DS or MAX_RT interrupt flag
	rfm70_chipDisable();
	value = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_REG|RFM70_REG_CONFIG); // read register CONFIG's value
	value = value | 0x01; //set bit 1
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, value); // Set PWR_UP bit, enable CRC(2 destinationBufferLength) & Prim:RX. RX_DR enabled..
	rfm70_chipEnable();
}
void rfm70_selectTransmitMode() {
	unsigned char value;
	rfm70_writeSingleByteToRegister(RFM70_CMD_FLUSH_TX, 0); // flush Tx
	rfm70_chipDisable();
	value = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_REG|RFM70_REG_CONFIG);
	value = value & 0xfe; //clear bit 0
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_CONFIG, value);
	rfm70_chipEnable();
}
void rfm70_selectBank(char _cfg) {
	unsigned char tmp;
	tmp = rfm70_readSingleByteFromRegister(7);
	tmp = tmp & 0x80;
	if( ( tmp && _cfg==0 ) || ( tmp==0 && _cfg ) ) rfm70_writeSingleByteToRegister(RFM70_CMD_ACTIVATE, 0x53);
}
void rfm70_selectChannel(unsigned char newChannel) {
	rfm70_writeSingleByteToRegister((unsigned char) (RFM70_CMD_WRITE_REG|RFM70_REG_RF_CH), (unsigned char) newChannel);
}
void rfm70_selectRfTxPower(unsigned char txPower) {
	unsigned char i;
	if (txPower>3) return;
	i = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_REG|RFM70_REG_RF_SETUP);
	i &= 0b11111001;
	i |= txPower<<1;
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_RF_SETUP, i);
}
void rfm70_selectPipeAddressWidth(unsigned char addressWidth) {
	if (addressWidth<1||addressWidth>3) return;
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_SETUP_AW, addressWidth);
}
void rfm70_setPipeAddress(unsigned char targetPipe, unsigned char *addressValue, unsigned char addressLength) {
	if (targetPipe<0x0A || targetPipe>0x10) return;
	rfm70_writeToRegisterFromBufferN((RFM70_CMD_WRITE_REG|targetPipe), addressValue, addressLength);
}

// ***************************************************************************
//                  RFM70 HIGH LEVEL COMMUNICATION FUNCTIONS
// ***************************************************************************
unsigned char rfm70_sendData(unsigned char *payloadPointer, size_t payloadSize) {
	unsigned char i;
	i = rfm70_readSingleByteFromRegister(RFM70_REG_STATUS);
	if (i & RFM70_IRQ_STATUS_TX_FULL) return 1;
	rfm70_writeToRegisterFromBufferN(RFM70_CMD_WRITE_TX_PAYLOAD_NOACK, payloadPointer, payloadSize);
	return 0;
}
unsigned char rfm70_readFromFifoIntoBuffer(unsigned char *destinationBuffer, unsigned char maxReadCharacters){
	unsigned char i;
	// Is Data in FIFO?
	i = rfm70_readSingleByteFromRegister(RFM70_REG_FIFO_STATUS);
	if ( i&RFM70_FIFO_STATUS_RX_EMPTY ) return 0; // No Data in FIFO
	// Which Pipe received Data?
	i = rfm70_getPipeNumberForFifoData();
	if (i>5) return 0; // FIFO empty or invalid Byte in Pipe
	// How many bytes should we read?
	i = rfm70_readSingleByteFromRegister(RFM70_CMD_READ_RX_PAYLOAD_WIDTH);
	if (i==0 || i>32) return 0; // Invalid value or nothing received
	rfm70_readIntoBufferFromRegisterN(RFM70_CMD_READ_RX_PAYLOAD, destinationBuffer, min(i, maxReadCharacters));
	return i;
}
unsigned char rfm70_getPipeNumberForFifoData() {
	unsigned char i;
	i = rfm70_readSingleByteFromRegister(RFM70_REG_STATUS);
	i = (i&0b00001110)>>1;
	if (i==6) return 255;
	return i;
}

// ***************************************************************************
//                  RFM70 INIT FUNCTIONS
// ***************************************************************************
void rfm70_initPortsAndSpi() {
	// ***** Port declaration for SS, IRQ and CE lines
	PORTB.DIR = PIN3_bm | PIN1_bm | PIN0_bm; // 3[!SS]-OUT | 2[IRQ]-IN | 1[CE]-OUT | 0[NC]-OUT
	PORTB.OUT = PIN3_bm; // RFM Slave-Select Off;
	PORTB.PIN2CTRL = PORT_OPC_PULLUP_gc | PORT_ISC_FALLING_gc;

	// ***** Port declaration for SPI lines
	PORTC.DIR = PIN7_bm | PIN5_bm | PIN4_bm | PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm; // 7[SCK]-OUT | 6[MISO]-IN | 5[MOSI]-OUT | 4[HW!SS]-OUT (IMPORTANT!)
	PORTC.OUT = 0;

	// ***** SPI module init
	SPIC.CTRL = SPI_MASTER_bm | SPI_ENABLE_bm | SPI_MODE_0_gc | SPI_PRESCALER_DIV16_gc | SPI_CLK2X_bm;

	// ***** Interrupt Control
	PMIC.CTRL = PMIC_LOLVLEN_bm;
	PORTB.INT0MASK = PIN2_bm;
	PORTB.INTCTRL = PORT_INT0LVL_LO_gc;
}
void rfm70_initChipRegisters() {
	unsigned char i;
	unsigned char WriteArr[12];
	_delay_ms(50);
	rfm70_selectBank(RFM70_SELECT_BANK_0);
	// Init Bank 0
	for(i=0; i<21; i++) rfm70_writeSingleByteToRegister( (RFM70_CMD_WRITE_REG|pgm_read_byte(&rfm70_initBank0_R0_R1D[i][0])), pgm_read_byte(&rfm70_initBank0_R0_R1D[i][1]));
	// Write DestinationRegister 10 - Rx0 address
	memcpy_P(WriteArr, RX0_Address, 5);
	rfm70_writeToRegisterFromBufferN( RFM70_CMD_WRITE_REG|RFM70_REG_RX_ADDR_P0 , &WriteArr[0], 5);
	rfm70_writeToRegisterFromBufferN( RFM70_CMD_WRITE_REG|RFM70_REG_TX_ADDR ,    &WriteArr[0], 5);
	// Write DestinationRegister 11 - Rx1 address
	memcpy_P(WriteArr, RX1_Address, 5);
	rfm70_writeToRegisterFromBufferN( RFM70_CMD_WRITE_REG|RFM70_REG_RX_ADDR_P1 , &WriteArr[0], 5);
	if(!rfm70_readSingleByteFromRegister(RFM70_REG_FEATURE)) rfm70_writeSingleByteToRegister(RFM70_CMD_ACTIVATE, 0x73);
	// Continue Bank 0 Init
	rfm70_writeSingleByteToRegister( RFM70_CMD_WRITE_REG|pgm_read_byte(&rfm70_initBank0_R0_R1D[22][0]), pgm_read_byte(&rfm70_initBank0_R0_R1D[22][1]) );
	rfm70_writeSingleByteToRegister( RFM70_CMD_WRITE_REG|pgm_read_byte(&rfm70_initBank0_R0_R1D[21][0]), pgm_read_byte(&rfm70_initBank0_R0_R1D[21][1]) );
	//********************Write Bank1 register******************
	rfm70_selectBank(RFM70_SELECT_BANK_1);
	for(i=0;i<14;i++) {
		memcpy_P(WriteArr, rfm70_initBank1_R0_R0D[i], 4);
		rfm70_writeToRegisterFromBufferN((RFM70_CMD_WRITE_REG|i), &WriteArr[0], 4);
	}
	memcpy_P(WriteArr, rfm70_initBank1_R0E, 11);
	rfm70_writeToRegisterFromBufferN((RFM70_CMD_WRITE_REG|14), &WriteArr[0], 11);
	_delay_ms(50);
	//********************switch back to Bank0 register access******************
	rfm70_selectBank(RFM70_SELECT_BANK_0);
}

// ***************************************************************************
//                  RFM70 SPI HARDWARE FUNCTIONS
// ***************************************************************************
unsigned char rfm70_spi_rw(unsigned char value) {
	SPIC.DATA = value;
	while (!(SPIC.STATUS & SPI_IF_bm));
	return SPIC.DATA;
}
