﻿/*
 * PROJEKT: WAmbiLight (a.k.a. AmbiLight 3)
 * ========================================
 *
 *   Created: 08.11.2012 17:30:00
 *    Author: Christoph Söllner
 *
 */ 

/***********************************
 ** INCLUDES                      **
 ***********************************/
#include "config.h"
#include "cs_rfm70.h"
#include "flash.h"

/***********************************
 ** GLOBAL VARIABLES              **
 ***********************************/
unsigned char rxBaseAddress[5] = { RFM70_RX_BASE_ADDRESS };
unsigned char rxBaseAddressLsbNew = 0x00;
volatile unsigned char rxCompleteInterrupt = 0;
unsigned char rxBuffer[30];

/***********************************
 ** MAIN FUNCTION                 **
 ***********************************/
int main(void) {
	unsigned char i;
	
	// ******* PORT? INIT
	PORTA.DIR = PIN0_bm;
	PORTA.PIN1CTRL = PORT_OPC_PULLUP_gc; // Address Input C
	PORTA.PIN2CTRL = PORT_OPC_PULLUP_gc; // Address Input B
	PORTA.PIN3CTRL = PORT_OPC_PULLUP_gc; // Address Input A

	PORTC.OUTCLR = PIN4_bm | PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;
	PORTC.DIRSET = PIN4_bm | PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;

	PORTD.OUTCLR = PIN5_bm | PIN4_bm | PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;
	PORTD.DIRSET = PIN5_bm | PIN4_bm | PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;

	PORTE.OUTCLR = PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;
	PORTE.DIRSET = PIN3_bm | PIN2_bm | PIN1_bm | PIN0_bm;

	PORTR.DIRSET = PIN1_bm | PIN0_bm; // LEDs

	// ******* CLOCK INIT
	OSC.CTRL   = (1<<OSC_RC32MEN_bp);
	while (!(OSC.STATUS & OSC_RC32MRDY_bm));
	CCP        = CCP_IOREG_gc;         // Prepare configuration change
	CLK.CTRL   = CLK_SCLKSEL_RC32M_gc; // Set Clock Source to internal 32MHz Osc
	CCP        = CCP_IOREG_gc;         // Prepare configuration change
	CLK.PSCTRL = CLK_PSBCDIV_1_1_gc;   // Set Clock Division to 1, means full 32MHz Osc frequency
	led_error_off();
	
	// ******* TIMER INIT
	TCC0.CTRLA = TC_CLKSEL_DIV4_gc;
	TCC0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCC0.PER   = 0xFFFF;
	TCC1.CTRLA = TC_CLKSEL_DIV4_gc;
	TCC1.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCC1.PER   = 0xFFFF;

	TCD0.CTRLA = TC_CLKSEL_DIV4_gc;
	TCD0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCD0.PER   = 0xFFFF;
	TCD1.CTRLA = TC_CLKSEL_DIV4_gc;
	TCD1.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCD1.PER   = 0xFFFF;

	TCE0.CTRLA = TC_CLKSEL_DIV4_gc;
	TCE0.CTRLB = TC_CCDEN_bm | TC_CCCEN_bm | TC_CCBEN_bm | TC_CCAEN_bm | TC_WGMODE_SS_gc;
	TCE0.PER   = 0xFFFF;

	// ******* RFM Init
	rfm70_initPortsAndSpi();
	rfm70_initChipRegisters();
	rfm70_selectChannel(RFM70_RX_CHANNEL);
	rfm70_setPipeAddress(RFM70_REG_RX_ADDR_P0, rxBaseAddress, sizeof(rxBaseAddress));
	rfm70_selectReceiveMode();

	// ****** INIT OF MCU AND PERIPHERALS DONE
	for (i=0;i<3;i++) {
		// flash green LED thrice
		led_ok_on();
		_delay_ms(10);
		led_ok_off();
		_delay_ms(200);
	}
	_delay_ms(500);

	sei();

    while(1) {
		// TASK: Read Address nibble and write new Address if changed.
		rxBaseAddressLsbNew = (((~PORTA_IN)&0b00001110)>>1);
		if (rxBaseAddress[4] != rxBaseAddressLsbNew) {
			led_ok_on();
			_delay_ms(500);
			rxBaseAddressLsbNew = (((~PORTA_IN)&0b00001110)>>1);
			rxBaseAddress[4] = rxBaseAddressLsbNew;
			rfm70_setPipeAddress(RFM70_REG_RX_ADDR_P0, &rxBaseAddress[0], 5);
			led_ok_off();
		}

		// TASK: Receive new Frame, if RFM70 has a new one stored
		if (rxCompleteInterrupt) {
			led_warn_on();
			rxCompleteInterrupt = 0;
			// POWER DOWN to read Registers
			//   Truely resets RX_FIFO, otherwise Pakets with exact same payload
			//   (each bit) would not be received twice, see datasheet for this behaviour
			i  = rfm70_readSingleByteFromRegister(RFM70_REG_CONFIG);
			rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_CONFIG, i & 0b11111101);
			rfm70_readFromFifoIntoBuffer(rxBuffer, 15); // Read new Frame into RX-Buffer
			rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_STATUS, 0b01000000);
			rfm70_writeSingleByteToRegister(RFM70_CMD_FLUSH_RX, 0);
			//Power Up again
			rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_CONFIG, i | 0b00000010);
			_delay_ms(1); // Delay just to make LED light visible
			led_error_off();
			led_warn_off();
		}

		// TASK: Update Buffered TCC*-Register with PWM
		//       Layout R0R1R2R3R4G0G1G2G3G4B0B1B2B3B4
		TCE0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[ 0]]); // C1:RED
		TCE0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[ 5]]); // C1:GREEN
		TCE0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[10]]); // C1:BLUE

		TCD1_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 1]]); // C2:RED
		TCE0_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 6]]); // C2:GREEN
		TCD1_CCBBUF = pgm_read_word(&gamma22[rxBuffer[11]]); // C2:BLUE

		TCD0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[ 2]]); // C3:RED
		TCD0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[ 7]]); // C3:GREEN
		TCD0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[12]]); // C3:BLUE

		TCC1_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 3]]); // C4:RED
		TCD0_CCABUF = pgm_read_word(&gamma22[rxBuffer[ 8]]); // C4:GREEN
		TCC0_CCDBUF = pgm_read_word(&gamma22[rxBuffer[13]]); // C4:BLUE

		TCC0_CCBBUF = pgm_read_word(&gamma22[rxBuffer[ 4]]); // C5:RED
		TCC0_CCCBUF = pgm_read_word(&gamma22[rxBuffer[ 9]]); // C5:GREEN
		TCC0_CCABUF = pgm_read_word(&gamma22[rxBuffer[14]]); // C5:BLUE
    }
}

/***********************************
 ** INTERRUPT SERVICE ROUTINES    **
 ***********************************/
ISR(PORTA_INT0_vect) {
	// Notify main() of pending new frame
	rxCompleteInterrupt = 1;
}