﻿/*
 * USBDongle.c
 *
 * Created: 06.11.2012 16:13:11
 *  Author: Christoph Söllner
 */ 

#include "config.h"
#include "cs_rfm70.h"

#include "Descriptors.h"
#include <LUFA/Drivers/USB/USB.h>

// ********************************
// ** LOKALE VARIABLEN
// ********************************
volatile unsigned char txComplete=0;
         unsigned char usbRxBuffer[120];
volatile unsigned char usbRxBufferIndex;
volatile   signed char txDeviceIndex=-1;
           signed int  tmpChar;
         unsigned char rfm70_destinationBaseAddress[5] = { RFM70_TX_BASE_ADDRESS }; // [4] will be increased, [0] to [3] can be changed

// Define USB-Device-Description
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface = {
	.Config = {
		.ControlInterfaceNumber   = 0,
		.DataINEndpoint           = {
			.Address          = CDC_TX_EPADDR,
			.Size             = CDC_TXRX_EPSIZE,
			.Banks            = 1,
		},
		.DataOUTEndpoint = {
			.Address          = CDC_RX_EPADDR,
			.Size             = CDC_TXRX_EPSIZE,
			.Banks            = 1,
		},
		.NotificationEndpoint = {
			.Address          = CDC_NOTIFICATION_EPADDR,
			.Size             = CDC_NOTIFICATION_EPSIZE,
			.Banks            = 1,
		},
	},
};

/********************************
 ** MAIN FUNCTION             ***
 ********************************/
int main(void) {
	unsigned char i;

	// ****** PORT INIT LEDs
	DDRD = PIN0_bm | PIN1_bm | PIN2_bm;
	PORTD = 0;

	// ****** DEBUG OUTPUT GO
	led_error_on();
	led_warn_on();
	led_ok_on();
	
	// ****** MCU-CLOCK INIT
	CLKSEL0 = (0<<RCE) | (1<<EXTE) | (1<<CLKS);
	CLKPR   = (1<<CLKPCE);
	CLKPR   = (0<<CLKPS3) | (0<<CLKPS2) | (0<<CLKPS1) | (0<<CLKPS0);
	// ****** TIMER INIT
	TCCR1A  = (0<<COM1A1)|(0<<COM1A0) | (0<<COM1B1)|(0<<COM1B0) | (0<<COM1C1)|(0<<COM1C0) | (0<<WGM11)|(0<<WGM10);
	TCCR1B  = (0<<ICNC1) | (0<<ICES1) | (0<<WGM13)|(0<<WGM12) | (0<<CS12)|(1<<CS11)|(0<<CS10); // TC1: 1 MHz Clock-Cycle. For Frame-Sync make sure it overflows every 10000 TimerTicks
	TCNT1   = (65535-10000);
	TIMSK1  = (1<<TOIE1);
	led_ok_off();

	// ****** USB INIT
	USBCON = 0;
	_delay_us(50);
	USBCON = (1<<USBE);
	_delay_us(50);
	USB_Init();
	led_warn_off();

	// ****** RFM INIT
	rfm70_initPortsAndSpi();
	rfm70_initChipRegisters();
	rfm70_selectChannel(RFM70_TX_CHANNEL);
	rfm70_selectTransmitMode();
	led_error_off();

	// ****** INIT OF MCU AND PERIPHERALS DONE
	for (i=0;i<3;i++) {
		led_ok_on();
		_delay_ms(10);
		led_ok_off();
		_delay_ms(200);
	}
	_delay_ms(500);

	sei();


	while(1) {
		// TASK: When data available READ 120 Bytes FROM USB 
		// TASK: When Timeout, Clear RX_INDEX
		// TASK: When RX_INDEX==120, send data to devices, 0-14 to ???000, 15-29 to ???001, etc.

		// Is Character available in USB-Input Buffer?
		tmpChar = -1;
		if (CDC_Device_BytesReceived(&VirtualSerial_CDC_Interface)>0) {
			// Timer Reset
			TCNT1 = (65535-10000);
			TCCR1B = 2;
			// Read one Character, unfortunately no Burst-Reading available
			tmpChar = CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);
		}

		// Yes, valid character received
		if (tmpChar>=0) {
			// Write it into RX Buffer and incread index
			usbRxBuffer[usbRxBufferIndex] = (tmpChar&0xFF);
			usbRxBufferIndex++;
			// Ah, Frame complete, transmit to LED controllers
			if (usbRxBufferIndex==120) {
				led_warn_on();
				for (txDeviceIndex=0; txDeviceIndex<8; txDeviceIndex++) {
					// Compose destination address
					rfm70_destinationBaseAddress[4] = txDeviceIndex;
					// Set destination address
					rfm70_setPipeAddress(RFM70_REG_TX_ADDR, rfm70_destinationBaseAddress, 5);
					// Send data and receive status code, 0 mean no error
					if (rfm70_sendData(usbRxBuffer+(txDeviceIndex*15), 15)) led_error_on();
					// Wait for RFM70 TX_COMPLETE interrupt to fire (see ISR below)
					while(!txComplete);
					txComplete=0;
				}
				// Flush/clear USB hardware buffer
				while (CDC_Device_BytesReceived(&VirtualSerial_CDC_Interface)) { tmpChar = CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface); }
				// Clear RX buffer for next frame. Propably not necessary though.
				for (i=0;i<120;i++) usbRxBuffer[i]=0;
				usbRxBufferIndex=0;
				led_error_off();
				led_warn_off();
			}
		}
		// Call functions that allow device to be polled
		CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
		USB_USBTask();
	}

}

void EVENT_USB_Device_Connect(void) {
	// When connected, i.e. COM? shows up (not opened yet)
	led_ok_on();
}
void EVENT_USB_Device_Disconnect(void) {
	// When disconnected, only under linux will device be disconnected when not needed
	led_ok_off();
}
void EVENT_USB_Device_ConfigurationChanged(void) {
	// Show USB enumeration error with red LED
	bool ConfigSuccess = true;
	ConfigSuccess &= CDC_Device_ConfigureEndpoints(&VirtualSerial_CDC_Interface);
	if (ConfigSuccess) {
		led_error_off();
	} else {
		led_error_on();
	}
}
void EVENT_USB_Device_ControlRequest(void) {
	// Do something USB specific
    CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
}

ISR(INT7_vect) {
	// RFM70 configured to fire TX_COMPLETE to this pin
	// Notify main (which is busy-waiting) of completion
	txComplete=1;
	// Clear flag within RFM70 so it can fire again
	rfm70_writeSingleByteToRegister(RFM70_CMD_WRITE_REG|RFM70_REG_STATUS, RFM70_IRQ_STATUS_TX_DS);
}

ISR(TIMER1_OVF_vect) {
	// syncs to PC Host via resetting USB rx buffer for a new frame every 10 ms (at most) after last character received.
	TCCR1B = 0;
	usbRxBufferIndex=0;
}